@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2016, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2017, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -25,6 +25,7 @@
 
 char *data_to_hex(char *data, size_t len);
 void logmsg(const char *msg, ...);
+long timediff(struct timeval newer, struct timeval older);
 
 #define TEST_DATA_PATH "%s/data/test%ld"
 
@@ -36,16 +37,16 @@ extern const char *path;
 /* global variable, log file name */
 extern const char *serverlogfile;
 
-#ifdef WIN32
+#if defined(WIN32) || defined(_WIN32)
 #include <process.h>
 #include <fcntl.h>
 
-#define sleep(sec)   Sleep ((sec)*1000)
+#define sleep(sec) Sleep ((sec)*1000)
 
 #undef perror
 #define perror(m) win32_perror(m)
-void win32_perror (const char *msg);
-#endif  /* WIN32 */
+void win32_perror(const char *msg);
+#endif  /* WIN32 or _WIN32 */
 
 #ifdef USE_WINSOCK
 void win32_init(void);
